from re import fullmatch

from django.forms import ValidationError
from django.utils.translation import gettext as _
from dynamic_preferences.preferences import Section
from dynamic_preferences.registries import global_preferences_registry
from dynamic_preferences.types import BooleanPreference, StringPreference

mailing_section = Section("wbmailing")


@global_preferences_registry.register
class DefaultSourceMailPreference(StringPreference):
    section = mailing_section
    name = "default_source_mail"
    default = "info@stainly-bench.com"

    verbose_name = _("Default Source Mail Preference")
    help_text = _("The default address used to send emails from")

    def validate(self, value):
        if not fullmatch(r"[^@]+@[^@]+\.[^@]+", value):
            raise ValidationError(_("Not a valid email format"))


@global_preferences_registry.register
class AutomaticallyApproveUnsubscriptionRequestFromHardBound(BooleanPreference):
    section = mailing_section
    name = "automatically_approve_unsubscription_request_from_hard_bounce"
    default = False

    verbose_name = _("Automatically approve unsubscription request from hard bounce")
    help_text = _(
        "Automatically approve unsubscription request from hard bounce received from the ESP tracking system"
    )



from .mailing_lists import (
    MailingListEmailContactThroughModelModelSerializer,
    MailingListEntryRepresentationSerializer,
    MailingListListSerializer,
    MailingListModelSerializer,
    MailingListRepresentationSerializer,
    MailingListSubscriberChangeRequestModelSerializer,
)
from .mails import (
    MailEventModelSerializer,
    MailModelSerializer,
    MailRepresentationSerializer,
    MailStatusMassMailModelSerializer,
    MailTemplateModelSerializer,
    MailTemplateRepresentationSerializer,
    MassMailListSerializer,
    MassMailModelSerializer,
    MassMailRepresentationSerializer,
)



from django_celery_beat.models import IntervalSchedule, PeriodicTask
from django.db import DEFAULT_DB_ALIAS
from django.apps import apps as global_apps


def initialize_task(app_config, verbosity=2, interactive=True, using=DEFAULT_DB_ALIAS, apps=global_apps, **kwargs):
    PeriodicTask.objects.update_or_create(
        task="wbmailing.tasks.periodic_send_mass_mail_as_tasks",
        defaults={
            "name": "Mailing: Periodically send scheduled mass mails",
            "interval": IntervalSchedule.objects.get_or_create(every=120, period=IntervalSchedule.SECONDS)[0],
            "crontab": None,
        },
    )
    PeriodicTask.objects.update_or_create(
        task="wbmailing.tasks.check_and_remove_expired_mailinglist_subscription",
        defaults={
            "name": "Mailing: Remove expired contact from mailing list",
            "interval": IntervalSchedule.objects.get_or_create(every=1, period=IntervalSchedule.DAYS)[0],
            "crontab": None,
        },
    )
